# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)
# $HeadURL: https://svn.base-art.net/full/xmlobject/trunk/XMLObject/Node.py $
# $Id: Node.py 19 2004-08-15 15:47:30Z test $



class RequiredNodeError(Exception):

    def __init__(self, name,xmlData=''):
        self._xml = xmlData
        self._msg = "'%s' attribute is not mandatory" % name
        if self._xml:
            self._msg += " in '%s'" % self._xml

    def __str__(self):
        return self._msg

    
class Node(object):
    """ This is the base class for XMLObject's content. Each node has:

        - a `type` (string)
        - an `itemType` (string) : used by ListNode so that it remembers
          what kind of items it has to store.
        - a `default` value
        - a `optional` boolean switch
        - a `noLimit` boolean switch
        
        For each of these properties, there are getters and setters.
        * `xmlrepr` is the method responsible of giving a string representing
          the node. User defined nodes should override it.
        * `getValueFromDom` is used to build a Node given its DOM tree.
    """

    def __init__(self, type='', itemType='', default=None, optional=False, noLimit=False):
        self.setType(type)
        self.setItemType(itemType)
        self._defaultValue = default
        self.setValue(default)
        self.setName('')
        self.setOptional(optional)
        self.setNoLimit(noLimit)

    def setNoLimit(self, val):
        self._noLimit = val

    def isNoLimit(self):
        return self._noLimit

    def setOptional(self, val):
        self._optional = val

    def isOptional(self):
        return self._optional

    def setItemType(self, type):
        self._itemType = type

    def getItemType(self):
        return self._itemType
        
    def setName(self, name):
        self._nodeName = name

    def getName(self):
        return self._nodeName

    def setValue(self, value):
        self._value = value
        
    def getValue(self):
        return self._value

    def setType(self, type):
        self._type = type

    def getType(self):
        return self._type

    def getParentType(self):
        return self._parentType

    def setParentType(self, parentType):
        self._parentType = parentType

    def xmlrepr(self, parentInstance=None):
        return ''

    def getValueFromDom(self, dom, attrName, **kw):
        return None

    def checkType(self, val):
        return True

    def resetValue(self):
        self.setValue(self._defaultValue)
        

    def setRegistry(self, reg):
        self._registry = reg

    def getRegistry(self):
        return self._registry
    
