# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)

import sys
sys.path.append('..')

from XMLObject import *

## Snippet "playlist-decl"
class Song(XMLObject):
    file = StringAttribute()
    length = IntegerAttribute()
    artist = StringAttribute(optional=True)
    title = TextNode(default = 'Track Name')
    comment = CommentNode(default = 'Blah blah')
    
class Playlist(XMLObject):
    _entities = [ ('&xml;','eXtensible Markup Langage')]
    name = StringAttribute()
    songs = ListNode('Song')
## End Snippet

## Snippet "playlist-creation"
s1 = Song(file='foobar.ogg', length=300, artist='foo', title='Bar')

pList = Playlist(songs=[s1])
pList.name = 'My Favorites in &xml;'
## End Snippet

## Snippet "playlist-newSong"
s2 = Song(file='opensource.ogg', comment='hey man it rocks')
s2.length = 250
pList.songs.append(s2)
## End Snippet

## Snippet "playlist-save"
xmlPlaylist = pList.toXml()

pList2 = Playlist.fromXml(xmlPlaylist)
print pList2.toXml()
assert xmlPlaylist == pList2.toXml(), 'Pb during import/export'
## End Snippet

"""
## Snippet "playlist-display"
<?xml version="1.0" encoding="iso-8859-1" ?>
<playlist name="My Favorites in eXtensible Markup Langage">
  <song artist="foo" file="foobar.ogg" length="300">
    <!-- Blah blah -->
    <title>
        Bar
    </title>
  </song>
  <song file="opensource.ogg" length="250">
    <!-- hey man it rocks -->
    </comment>
    <title>
        Track Name
    </title>
  </song>
</playlist>
## End Snippet
"""
