# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)

import sys
sys.path.append('..')

from XMLObject import *

## Snippet "xxx-decl"
class XXX(XMLObject):
    """
    <!ELEMENT XXX (AAA+ , BBB+)>
    """
    aaa = ListNode('AAA',optional=False)
    bbb = ListNode('BBB',optional=False)
    
class AAA(XMLObject):
    """
    <!ELEMENT AAA (BBB | CCC )>
    """
    mainNode = ChoiceNode(['BBB','CCC'])

class BBB(XMLObject):
    """
    <!ELEMENT BBB (#PCDATA | CCC )*>
    """
    mainNode = ChoiceNode(['#PCDATA', 'CCC'],optional=True, noLimit=True)

class CCC(XMLObject):
    """
    <!ELEMENT CCC (#PCDATA)>
    """
    mainNode = TextNode()
## End Snippet

## Snippet "xxx-mainNode"
cc = CCC('Some Text')
print cc.toXml(headers=False)
"""
<cCC>
    Some Text
</cCC>
"""
print cc.mainNode
"""
Some Text
"""
## End Snippet

## Snippet "xxx-invalid"
xx = XXX()
print xx.toXml()
"""
Traceback (most recent call last):
 ...
TypeError: Expected some node in 'XXX.aaa' as it is not optional
"""
## End Snippet

## Snippet "xxx-fill-in1"
xxx = XXX()
xxx.aaa.append(CCC('Precisely one element.'))

bb = BBB()
bb.append(CCC())
bb.append(CCC())
bb.append(CCC())
xxx.aaa.append(bb)

xxx.bbb.append(BBB())
## End Snippet
## Snippet "xxx-fill-in2"
bb2 = BBB()
bb2.append('This is')
bb2.append(CCC())
bb2.append('a combination')
bb2.append(CCC())
bb2.append('of')
bb2.append(CCC('CCC elements'))
bb2.append('and text')
bb2.append(CCC())
xxx.bbb.append(bb2)

xxx.bbb.append(BBB('Text only.'))
## End Snippet


xxxStr = xxx.toXml()
xxx2 = XXX.fromXml(xxxStr)
assert xxx2 == xxx, "Pb during import/export"

print xxxStr

"""
## Snippet "xxx-display"
<?xml version="1.0" encoding="iso-8859-1" ?>
<xXX>
  <aAA>
    <cCC>
        Precisely one element.
    </cCC>
  </aAA>
  <aAA>
    <bBB>
      <cCC/>
      <cCC/>
      <cCC/>
    </bBB>
  </aAA>
  <bBB/>
  <bBB>
      This is
    <cCC/>
      a combination
    <cCC/>
      of
    <cCC>
        CCC elements
    </cCC>
      and text
    <cCC/>
  </bBB>
  <bBB>
      Text only.
  </bBB>
</xXX>
## End Snippet
"""
